#!/bin/bash 

. ~/.profile.common
. ~/.cvspass

cd $(dirname $0)

# first check to see if a sync is already going on.  if it is, we do NOT
# want to sync calendars. 
SYNC=/dev/ttyUSB1
if [ -e "$SYNC" ]
then 
	SYNCING=1
else
	SYNCING=0
fi

if [ "$SYNCING" -eq 1 ]
then
	echo "Detected syncing going on.  Not going to update calendar now."
	exit 5
fi

# pidfile...
PIDFILE="/tmp/exchange_calendar_1way_sync.pid"

if [ -f "$PIDFILE" ]
then
	echo "Previous pidfile found."
	PID=$(cat $PIDFILE)
	COUNT=$(ps -aef | grep $PID | grep -v grep | wc -l)
	if [ $COUNT -gt 0 ]
	then
		echo "Found: [$COUNT] processes running with that pid."
		echo "Not running."
		exit 6
	else
		echo "No processes found for pid: [$PID]. continuing."
	fi
fi

# SAVE off our pid
echo "$$" > "$PIDFILE"

# should we be verbose with debugging?
DEBUG=0

MERGECAL="${HOME}/.kde/share/apps/korganizer/std.ics"
#MERGECAL="${HOME}/test-korganizer-std.ics"

# back it up!!!
cp "$MERGECAL" "$MERGECAL.$(date +'%Y%m%d%H')"

echo "Working on file: [$MERGECAL]"

# -- MAIN 
RUBX="ruby-exchange"

echo "Fetching calendar..."
cd "$RUBX" || exit 9

nice -n 20 ruby rexport.rb 

rc=$?
if [ $rc -ne 0 ]
then
	echo "Unable to fetch calendar. exiting."
	exit 1
fi

echo "Done fetching calendar..."

# grab a snapshot of our calendar file before start merging
BEGINMD5=$(md5sum "${MERGECAL}" | awk '{print $1}')

echo "Now merging calendars..."

WORKFILE="$HOME/.kde/share/apps/korganizer/std.ics.updated"

~/local/bin/mergecalendars \
	--korgfile $HOME/.kde/share/apps/korganizer/std.ics \
	--newfile $HOME/bin/ruby-exchange/exchange-calendar.ics

rc=$?
if [ $rc -ne 0 ] || [ ! -s "$WORKFILE" ]
then
	echo "Unable to merge calendars. exiting."
	exit 1
fi


# check the md5sum of the file now that we're done and ONLY overwrite it if
# it has not changed since we started
ENDMD5=$(md5sum "${MERGECAL}" | awk '{print $1}')

if [ "$BEGINMD5" = "$ENDMD5" ]
then
	echo "File same as when I started. overwriting it."
	cp "$WORKFILE" "$MERGECAL"
else
	echo "File has changed since I started. leaving it be."
fi

rm "$PIDFILE"
