#!/usr/bin/env ruby
require 'icalendar'
require 'date'

puts "Opening korganizer file..."
# Open a file or string to parse
korgfile = "#{ENV['HOME']}/.kde/share/apps/korganizer/std.ics"
calfilekorg = File.open(korgfile)

# Parser returns an array of calendars because a single file
# can have multiple calendar objects.
calskorg = Icalendar::parse(calfilekorg)
calkorg  = calskorg.first

numkorgstart = calkorg.events.size
puts "  - Opened korganizer file with #{numkorgstart} events."

puts "Opening exchange calendar..."
# Open a file or string to parse
calfilexchg = File.open("exchange-calendar.ics")

# Parser returns an array of calendars because a single file
# can have multiple calendar objects.
calsxchg = Icalendar::parse(calfilexchg)
calxchg  = calsxchg.first

numxchg = calxchg.events.size
puts "  - Opened exchange file with #{numxchg} events."

puts "Looking for previous pilot ids for exchange events..."
# Now iterate over each exchange event and try to find a previous pilot id
i = 0
ev = calxchg.events.each do |ev|
	#puts "i: #{i}, uid: #{ev.uid} summary: #{ev.summary}"
	evkorg = calkorg.find_event(ev.uid)
	if evkorg and evkorg.x_pilotid
		pilotid = evkorg.x_pilotid
		#puts "    - Pilotid: [#{pilotid}], summary: [#{evkorg.summary}]"
		ev.x_pilotid = pilotid
		ev.x_pilotstat = "1"
		i = i + 1
	end
end

numkorgpilotids = i
puts "  - Found #{numkorgpilotids} pilotids."

puts "Now searching for previous WorkXChange events in korganizer's calendar"
# Now iterate over each korganizer event and remove previous WorkXChange
# events
i = 0
ev = calkorg.events.reverse_each do |ev|
	#puts "  - uid: #{ev.uid}, category: #{ev.categories}."
	if ev.categories.to_s == "WorkXChange"
		#puts "    - Found one. uid: #{ev.uid}, summary: #{ev.summary}"
		calkorg.remove_component(ev)
		i = i + 1
	end
end

numkorgremoved = i
puts "  - Found #{numkorgremoved} prior WorkXChange category events."

puts "Merging in exchange events into korganizer calendar..."
# Now iterate over each exchange event 
i = 0
ev = calxchg.events.each do |ev|
	calkorg.add_component(ev)
end

puts "Ended up with #{calkorg.events.size} events in korganizer calendar."

puts "Saving updated korganizer file..."

out = File.new("korganizer-updated.ics", "w")
out.write(calkorg.to_ical)
out.close


puts "Saving updated exchange file..."

out = File.new("exchange-calendar-updated.ics", "w")
out.write(calxchg.to_ical)
out.close




